<?php

//  Finds the root files directory for the package based on the package's
//  machine name.

function package_builder_get_root_dir($node) {
  return file_directory_path() . '/packages/' . $node->machine_name;
}

//  Builds our directory structure
//  The supported types shouldn't be hardcoded here

function package_builder_setup_dirs($root_dir) {
  $subs = array('views', 'rules', 'flags', 'cck', 'roles', 'nodes', 'users', 'vardumps');
  if (!file_check_directory($root_dir)) {
    mkdir($root_dir, 0777, TRUE);
  }
  foreach ($subs as $sub) {
    $sub = $root_dir . '/' . $sub;
    if (!file_check_directory($sub)) {
      mkdir($sub, 0777, TRUE);
    }
  }
}

//  This sets up the package on export.
//  It moves the generic files and does the find/replace

function package_builder_setup_files($root_dir, $node, $mod_list) {
  if (!$mod_list) {
    $mod_list = array();
  }

  $mod_path = drupal_get_path('module', 'package_builder');

  $text = file_get_contents($mod_path . '/generic.info.inc');
  $text = str_replace('NAME', $node->title, $text);
  $text = $text . "#  Manually Added Dependencies\n";
  foreach ($mod_list as $mod) {
    $text = $text . 'dependencies[] = ' . $mod . "\n";
  }
  file_save_data($text, $root_dir . '/' . $node->machine_name . '.info',
                 FILE_EXISTS_REPLACE);


  $text = file_get_contents($mod_path . '/generic.install.inc');
  $text = str_replace('NAME', $node->machine_name, $text);
  file_save_data($text, $root_dir . '/' . $node->machine_name . '.install',
                 FILE_EXISTS_REPLACE);

  $start =  $mod_path . '/generic.module.inc';
  $dest  = $root_dir . '/' . $node->machine_name . '.module';
  file_copy($start, $dest, FILE_EXISTS_REPLACE);

  $start = $mod_path . '/install.inc';
  $dest  = $root_dir . '/install.inc';
  file_copy($start, $dest, FILE_EXISTS_REPLACE);

  $start = $mod_path . '/uninstall.inc';
  $dest  = $root_dir . '/uninstall.inc';
  file_copy($start, $dest, FILE_EXISTS_REPLACE);
}

//  This just empties a directory.
//  It only deletes the files we're looking for, just in case someone
//  stuck something random in there.

function package_builder_clear_dir($dir, $ext) {
  foreach (glob($dir . '*' . $ext) as $file) {
    file_delete($file);
  }
}

//  Takes a list of roles, exports them, and writes them to a file.

function package_builder_export_roles($rids, $root_dir) {
  package_builder_clear_dir($root_dir . '/roles/', '.role');
  if (!$rids) {
    return;
  }
  foreach ($rids as $rid) {
    $role  = db_fetch_object(db_query("SELECT * FROM {role} WHERE rid = %d", $rid));
    $perms = permissions_get_permissions_for_role($role->name);
    $text = var_export(array('permissions' => $perms,
                             'name' => $role->name), TRUE);
    $text  = "<?php\n \$role = " . $text . ';';
    file_save_data($text, $root_dir . '/roles/' . $role->name . '.role',
                   FILE_EXISTS_REPLACE);
  }
}

//  Takes a list of nodes, exports them, and writes them to a file.

function package_builder_export_nodes($nids = NULL, $root_dir) {
  package_builder_clear_dir($root_dir . '/nodes/', '.node');
  if (!$nids) {
    return;
  }
  foreach ($nids as $nid) {
    $node_ = node_load($nid);
    $title = $node_->title;
    unset($node_->nid);
    unset($node_->vid);
    $node_->uid = 1;
    $node_ = get_object_vars($node_);
    $text = "<?php\n \$node = " . var_export($node_, TRUE) . ';';
    file_save_data($text, $root_dir . '/nodes/' . $title . '.node',
                   FILE_EXISTS_REPLACE);
  }
}


//  Takes a list of content types, exports them, and writes them to a file.

function package_builder_export_content_types($types = NULL, $root_dir) {
  package_builder_clear_dir($root_dir . '/cck/', '.cck');
  if (!$types) {
    return;
  }
  foreach ($types as $type) {
    //work around bug in cck GLOBAL variables argasdgsalkfj
    unset($GLOBALS['content_copy']['submissions']);
    $form_values = array();
    $form_values['type_name'] = $type;
    $form_values['fields'] = content_copy_fields($type);
    $text = content_copy_export($form_values);
    file_save_data($text, $root_dir . '/cck/' . $type . '.cck', FILE_EXISTS_REPLACE);
    }
}

//  Takes a list of views, exports them, and writes them to a file.

function package_builder_export_views($view_names = NULL, $root_dir) {
  package_builder_clear_dir($root_dir . '/views/', '.view');
  if (!$view_names) {
    return;
  }
  $views = views_get_all_views();
  foreach ($view_names as $name) {
    file_save_data($views[$name]->export(),
                   $root_dir . '/views/' . $name . '.view', FILE_EXISTS_REPLACE);
  }
}

//  Takes a list of flags, exports them, and writes them to a file.

function package_builder_export_flags($names = NULL, $root_dir) {
  package_builder_clear_dir($root_dir . '/flags/', '.flag');
  if (!$names) {
    return;
  }
  require_once('./' . drupal_get_path('module', 'flag') . '/includes/flag.export.inc');
  foreach ($names as $name) {
    $text  = flag_export_flags(array($name));
    file_save_data($text, $root_dir . '/flags/' . $name . '.flag', FILE_EXISTS_REPLACE);
  }
}

//  Takes a list of rules, exports them, and writes them to a file.

function package_builder_export_rules($rules = NULL, $root_dir, $machine_name) {
  package_builder_clear_dir($root_dir . '/rules/', '.rule');
  if (!$rules) {
    return;
  }
  require_once(drupal_get_path('module', 'rules') . '_admin/rules_admin.export.inc');
  foreach ($rules as $rule_) {
    $form_state['values']['export']['rules'] = array($rule_ => $rule_);
    rules_admin_form_export_submit(array(), $form_state);
    $name = key($form_state['export']['rules']);
    $label = $form_state['export']['rules'][$name]['#label'];
    $text = var_export($form_state['export'], TRUE);
    $text = str_replace($name, $label, $text);
    $category = implode(',', $form_state['export']['rules'][$name]['#categories']);
    $label = str_replace(' ', '_', $label);
    if ($category) {
      file_save_data($text, $root_dir . '/rules/' . '('.$category.')' . $label . '.rule', FILE_EXISTS_REPLACE);
    }
    else {
      file_save_data($text, $root_dir . '/rules/' . $label . '.rule', FILE_EXISTS_REPLACE);
    }
  }
}

function package_builder_export_rules_sets($rules = NULL, $root_dir, $machine_name) {
  package_builder_clear_dir($root_dir . '/rules/', '.ruleset');
  if (!$rules) {
    return;
  }
  require_once(drupal_get_path('module', 'rules') . '_admin/rules_admin.export.inc');
  foreach ($rules as $rule_) {
    $form_state['values']['export']['rule_sets'] = array($rule_ => $rule_);
    rules_admin_form_export_submit(array(), $form_state);
    $name = key($form_state['export']['rule_sets']);
    $label = $form_state['export']['rule_sets'][$name]['label'];
    $text = var_export($form_state['export'], TRUE);
    $category = implode(',', $form_state['export']['rule_sets'][$name]['categories']);
    foreach ($form_state['export']['rule_sets'] as $key => $value) {
      $text = str_replace($key, $value['label'], $text);
    }
    $label = str_replace(' ', '_', $label);
    if ($category) {
      file_save_data($text, $root_dir . '/rules/' . '('.$category.')' . $label . '.ruleset', FILE_EXISTS_REPLACE);
    }
    else {
      file_save_data($text, $root_dir . '/rules/' . $label . '.ruleset', FILE_EXISTS_REPLACE);
    }
  }
}


function package_builder_export_users($users = NULL, $root_dir) {
  package_builder_clear_dir($root_dir . '/users/', '.user');
  if (!$users) {
    return;
  }
  foreach ($users as $user_) {
    $user_ = user_load($user_);
    $data = $user_->data;
    $code = md5(uniqid(mt_rand(), true));
    $code = substr($code, 0, 10);
    $user_->pass = $code;
    unset($user_->data);
    unset($user_->uid);
    unset($user_->form_build_id);
    $user_ = get_object_vars($user_);
    $text = "<?php\n \$user_ = " . var_export($user_, TRUE) . ';';
    file_save_data($text, $root_dir . '/users/' . $user_['name'] . '.user', FILE_EXISTS_REPLACE);
  }
}


function package_builder_export_vardumps($vardumps = NULL, $root_dir) {
  package_builder_clear_dir($root_dir . '/vardumps/', '.dump');
  if (!$vardumps) {
    return;
  }
  foreach ($vardumps as $vardump) {
    $vardump = node_load($vardump);
    $title = $vardump->title;
    $text = $vardump->body;
    file_save_data($text, $root_dir . '/vardumps/' . $title . '.dump', FILE_EXISTS_REPLACE);
  }
}
